/* Target-dependent code for Juniper on ARM processors.
   Copyright 2004 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */


#include "defs.h"
#include "gdb_string.h"
#include "gdbarch.h"
#include "gdbcore.h"
#include "arm-tdep.h"
#include "regcache.h"
#include "bfd.h"
#include "elf-bfd.h"

/*
 * Ripped off from NetBSD
 */
struct reg {
	unsigned int r[13];
	unsigned int r_sp;
	unsigned int r_lr;
	unsigned int r_pc;
	unsigned int r_cpsr;
};

/*
 * An extended precision floating point number
 */

typedef struct fp_extended_precision {
	u_int32_t fp_exponent;
	u_int32_t fp_mantissa_hi;
	u_int32_t fp_mantissa_lo;
} fp_extended_precision_t;

typedef struct fp_extended_precision fp_reg_t;

struct fpreg {
	unsigned int fpr_fpsr;
	fp_reg_t fpr[8];
};

static void
arm_jnpr_core_read_registers (char *sect, unsigned size, int which,
                              CORE_ADDR addr)
{
  struct gdbarch_tdep *tdep = gdbarch_tdep (current_gdbarch);

  if (which == 0 && size == sizeof (struct reg)) /* Juniper arm ukern */
    {
      int reg;

      for (reg = 0; reg <= 15; ++reg)
	regcache_raw_supply (current_regcache, reg, sect + 4 * reg);

      regcache_raw_supply (current_regcache, ARM_PS_REGNUM, sect + 4 * 16);
    }
  else
    warning ("unrecognized regset %d in core file", which);
}

/* Core sniffer to tell FreeBSD and embedded core files apart */
static int
jnpr_core_sniffer (struct core_fns *our_fns, bfd *abfd)
{
  int result;

  result = (bfd_get_flavour (abfd) == our_fns -> core_flavour &&
	    (elf_elfheader(abfd)->e_ident[EI_OSABI] == ELFOSABI_NONE ||
	     elf_elfheader(abfd)->e_ident[EI_OSABI] == ELFOSABI_STANDALONE));
  return (result);
}

static struct core_fns arm_jnpr_core_fns =
  {
    bfd_target_elf_flavour,     /* core_flavour */
    default_check_format,       /* check_format */
    jnpr_core_sniffer,          /* core_sniffer */
    arm_jnpr_core_read_registers, /* core_read_registers */
    NULL                        /* next */
  };

void
_initialize_arm_jnpr ()
{
  deprecated_add_core_fns (&arm_jnpr_core_fns);
}
